#region License Information
// Copyright (c) 2005 - 2009, Allan Spartacus Mangune (allan@owlpal.com)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met:
//
// * Redistributions of source code must retain the above copyright notice, this list 
//   of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright notice, this list of 
//   conditions and the following disclaimer in the documentation and/or other materials provided 
//   with the distribution.
//
// * Neither the name of Owlpal nor the names of its contributors may be used to endorse or 
//   promote products derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
// BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
// OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Xml;

namespace Owlpal.Utils.Syndication
{
    /// <summary>
    /// 
    /// </summary>
    public class ReallySimpleSyndication
    {
        XmlTextWriter xmlTextWriter = new XmlTextWriter(HttpContext.Current.Response.OutputStream, Encoding.UTF8);
        private ChannelElementCollection channelElements = new ChannelElementCollection();
        private List<ChannelElementCollection> items = new List<ChannelElementCollection>();

        /// <summary>
        /// 
        /// </summary>
        public ChannelElementCollection ChannelElements
        {
            get { return this.channelElements; }
            set { this.channelElements = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        public List<ChannelElementCollection> Items
        {
            get { return this.items; }
            set { this.items = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        public void Feed()
        {
            HttpContext.Current.Response.Clear();
            HttpContext.Current.Response.ContentType = "text/xml";
            xmlTextWriter.WriteStartDocument();
            xmlTextWriter.WriteStartElement("rss");
            xmlTextWriter.WriteAttributeString("version", "2.0");
            xmlTextWriter.WriteStartElement("channel");

            BindChannelElements(this.channelElements);
            
            for (int i = 0; i < items.Count; i++)
            {
                xmlTextWriter.WriteStartElement("item");
                this.BindChannelElements(items[i]);
                xmlTextWriter.WriteEndElement();
            }
            
            xmlTextWriter.WriteEndElement();
            xmlTextWriter.WriteEndElement();
            xmlTextWriter.WriteEndDocument();
            xmlTextWriter.Flush();
            xmlTextWriter.Close();
            HttpContext.Current.Response.End();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="collection"></param>
        private void BindChannelElements(ChannelElementCollection collection)
        {
            collection.Required.Bind();
            this.WriteRequiredChannelElements(collection.Required);
            collection.Optional.Bind();
            this.WriteOptionalChannelElements(collection.Optional);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="optionalChannelElements"></param>
        private void WriteOptionalChannelElements(OptionalRSSChannelElements optionalChannelElements)
        {
            string[] attributes = {"language", "copyright", "docs", "generator", "ttl", 
                "managingEditor", "webMaster", "pubDate", "lastBuildDate", "category", "cloud", "image", 
                "rating", "textInput", "skipHours", "skipDays"};
            
            foreach (string attribute in attributes)
            {
                if (optionalChannelElements[attribute].Length != 0)
                    xmlTextWriter.WriteElementString(attribute, optionalChannelElements[attribute]);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="requiredChannelElements"></param>
        private void WriteRequiredChannelElements(RequiredRSSChannelElements requiredChannelElements)
        {
            string[] attributes = { "title", "link", "description" };
            
            foreach (string attribute in attributes)
            {
                if (requiredChannelElements[attribute].Length != 0)
                    xmlTextWriter.WriteElementString(attribute, requiredChannelElements[attribute]);
            }
        }
    }
}
